package com.yeejoin.amos.iot.business.queue.handler;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.entity.Alarm;
import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpGraphNodeBo;
import com.yeejoin.amos.iot.business.enums.AlarmLevel;
import com.yeejoin.amos.iot.business.enums.AlarmStatus;
import com.yeejoin.amos.iot.business.enums.AlarmType;
import com.yeejoin.amos.iot.business.queue.AlarmDataObj;
import com.yeejoin.amos.iot.business.queue.EqpConnData;
import org.apache.log4j.Logger;

import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 设备连通性告警处理
 *
 * @author as-lichenglong
 */
public class EqpConnAlarmHandler extends AbstractDataAlarm implements
        IDataAlarm {

    private static final Logger logger = Logger.getLogger(EqpConnAlarmHandler.class);

    private Map<String, Boolean> lastAlarmOfPortMap = new ConcurrentHashMap<>();

    public EqpConnAlarmHandler() {
        registerService();
    }

    @Override
    public void handData(AlarmDataObj alarmDataObj) {
        Object obj = alarmDataObj.getDataObj();
        EqpConnData eqpConnData = (EqpConnData) obj;
        Equipment equipment = deviceService.findEquipmentById(eqpConnData.getEqpId());
        raiseAlarm(eqpConnData, equipment);
    }

    /**
     * 产生告警
     *
     * @param eqpConnData
     * @param equipment
     */
    private void raiseAlarm(EqpConnData eqpConnData, Equipment equipment) {
        // 从缓存中取出上次的告警标志
        String lastAlarmFlagKey = eqpConnData.getProtocolName() + IotConstant.EquipmentConstant.UNDERLINE + eqpConnData.getEqpId();
        Boolean lastAlarmFlag = lastAlarmOfPortMap.get(lastAlarmFlagKey);
        if (null == lastAlarmFlag) {
            // 判断上次是否产生告警
            lastAlarmFlag = false;
            lastAlarmOfPortMap.put(lastAlarmFlagKey, lastAlarmFlag);
        }
        try {
            if (eqpConnData.isConnected()) {
                // 产生清除告警
                if (lastAlarmFlag) {
                    buildClearAlarm(equipment, eqpConnData);
                    lastAlarmFlag = false;
                    lastAlarmOfPortMap.put(lastAlarmFlagKey, lastAlarmFlag);
                }
            } else {
                // 产生新告警
                buildAlarm(equipment, eqpConnData);
                lastAlarmFlag = true;
                lastAlarmOfPortMap.put(lastAlarmFlagKey, lastAlarmFlag);
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
        }
    }

    /**
     * 构建新告警
     *
     * @param equipment
     * @param eqpConnData
     */
    private void buildAlarm(Equipment equipment, EqpConnData eqpConnData) {
        if (equipment == null) {
            Logger.getLogger(this.getClass()).warn(
                    "没有添加设备，无法构建告警，设备equipment:" + equipment);
            return;
        }
        EqpGraphNodeBo eqpGraphNodeBo = findEqpGraphNodeByEqpId(equipment
                .getId());
        if (eqpGraphNodeBo == null) {
            Logger.getLogger(this.getClass()).warn(
                    "没有视图，无法构建告警，设备equipment:" + equipment);
            return;
        }
        Alarm alarm = new Alarm();
        alarm.setName(equipment.getName());
        Map<String, Object> attrMap = getEqpAttrMap(equipment);
        String ip = attrMap.get(IotConstant.IP) == null ? "" : attrMap.get(
                IotConstant.IP).toString();
        alarm.setDescr(ip + " " + eqpConnData.getProtocolName() + "不通！");
        alarm.setType(AlarmType.EXCEPTION.getValue());
        alarm.setSecondaryType(eqpConnData.getProtocolName());
        alarm.setLevel(AlarmLevel.MINOR.getValue());
        alarm.setStatus(AlarmStatus.RAISE.getValue());
        alarm.setRaiseTime(new Date());
        alarm.setCount(1);
        alarm.setEqpId(equipment.getId());
        alarm.setOrgCode(equipment.getOrgCode());
        alarm.setSubgraphId(eqpGraphNodeBo.getSubGraphId());
        alarmService.raise(alarm, false);
    }

    /**
     * 构建清除告警
     *
     * @param equipment
     * @param eqpConnData
     */
    private void buildClearAlarm(Equipment equipment, EqpConnData eqpConnData) {
        if (equipment == null) {
            Logger.getLogger(this.getClass()).warn(
                    "没有添加设备，无法构建告警，设备equipment:" + equipment);
            return;
        }
        Alarm alarm = new Alarm();
        alarm.setName(equipment.getName());
        Map<String, Object> attrMap = getEqpAttrMap(equipment);
        String ip = attrMap.get(IotConstant.IP) == null ? "" : attrMap.get(
                IotConstant.IP).toString();
        alarm.setDescr(ip + " " + eqpConnData.getProtocolName() + "已连通！");
        alarm.setType(AlarmType.EXCEPTION.getValue());
        alarm.setSecondaryType(eqpConnData.getProtocolName());
        alarm.setLevel(AlarmLevel.MINOR.getValue());
        alarm.setStatus(AlarmStatus.RAISE.getValue());
        alarm.setClearTime(new Date());
        alarm.setEqpId(equipment.getId());
        alarm.setOrgCode(equipment.getOrgCode());
        alarm.setClearInfo("设备 " + equipment.getName() + " " + eqpConnData.getProtocolName() + "已连通！");
        alarmService.raise(alarm, true);
    }

}
