package com.yeejoin.amos.iot.mongodb.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.yeejoin.amos.iot.business.param.AlarmPageParam;
import com.yeejoin.amos.iot.core.common.response.CommonPage;
import com.yeejoin.amos.iot.mongodb.MongoCondition;
import com.yeejoin.amos.iot.mongodb.MongoSort;
import com.yeejoin.amos.iot.mongodb.service.intfc.IMetricDataQueryService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.AggregationOperation;
import org.springframework.data.mongodb.core.aggregation.AggregationResults;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.regex.Pattern;

@Service
public class MetricDataQueryServiceImpl implements IMetricDataQueryService {

    @Autowired
    private MongoTemplate mongoTemplate;

    @Override
    public List<JSONObject> list(String collectionName, List<MongoCondition> conditions,
                                 Integer skip, Integer limit, List<MongoSort> sorts) {
        Query query = buildQueryCondition(conditions);
        if (skip != null && limit != null) {
            query.skip(skip).limit(limit);
        }
        if (sorts != null && sorts.size() > 0) {
            List<Sort.Order> orders = Lists.newArrayList();
            sorts.forEach(s -> {
                if (s.getSortType().equals("asc")) {
                    orders.add(new Sort.Order(Sort.Direction.ASC, s.getSortName()));
                } else if (s.getSortType().equals("desc")) {
                    orders.add(new Sort.Order(Sort.Direction.DESC, s.getSortName()));
                }
            });
            if (orders.size() > 0) {
                query.with(new Sort(orders));
            }
        }
        return mongoTemplate.find(query, JSONObject.class, collectionName);
    }

    @Override
    public JSONObject findOne(String collectionName, List<MongoCondition> conditions) {
        return mongoTemplate.findOne(buildQueryCondition(conditions), JSONObject.class, collectionName);
    }

    @Override
    public Long count(String collectionName, List<MongoCondition> conditions) {
        return mongoTemplate.count(buildQueryCondition(conditions), collectionName);
    }

    @Override
    public List<JSONObject> statistic(String collectionName, List<MongoCondition> conditions,
                                      String statisticType, String groupBy, String sumBy,
                                      List<MongoSort> sorts) {
        List<AggregationOperation> operations = Lists.newArrayList();
        if (!CollectionUtils.isEmpty(conditions)) {
            Criteria criteria = null;
            for (MongoCondition c : conditions) {
                if ("is".equals(c.getQueryType())) {
                    criteria = Criteria.where(c.getName()).is(c.getValue());
                } else if ("in".equals(c.getQueryType())) {
                    criteria = Criteria.where(c.getName()).in(c.getValue());
                } else if ("gt".equals(c.getQueryType())) {
                    criteria = Criteria.where(c.getName()).gt(c.getValue());
                } else if ("lt".equals(c.getQueryType())) {
                    criteria = Criteria.where(c.getName()).lt(c.getValue());
                } else if ("gte".equals(c.getQueryType())) {
                    criteria = Criteria.where(c.getName()).gte(c.getValue());
                } else if ("lte".equals(c.getQueryType())) {
                    criteria = Criteria.where(c.getName()).lte(c.getValue());
                } else if ("nin".equals(c.getQueryType())) {
                    criteria = Criteria.where(c.getName()).nin(c.getValue());
                } else if ("regex".equals(c.getQueryType())) {
                    criteria = Criteria.where(c.getName()).regex(c.getValue().toString());
                }
            }
            if (criteria != null) {
                operations.add(Aggregation.match(criteria));
            }
        }
        if (statisticType != null && groupBy != null) {
            if ("count".equals(statisticType)) {
                operations.add(Aggregation.group(groupBy).count().as(groupBy + "_" + statisticType));
            } else if ("sum".equals(statisticType) && sumBy != null) {
                operations.add(Aggregation.group(groupBy).sum(sumBy).as(sumBy + "_" + statisticType));
            }
        }
        if (sorts != null && sorts.size() > 0) {
            sorts.forEach(s -> {
                if (s.getSortType().equals("asc")) {
                    operations.add(Aggregation.sort(Sort.Direction.ASC, s.getSortName()));
                } else if (s.getSortType().equals("desc")) {
                    operations.add(Aggregation.sort(Sort.Direction.DESC, s.getSortName()));
                }
            });
        }
        Aggregation aggregation = Aggregation.newAggregation(operations);
        AggregationResults<JSONObject> out = mongoTemplate.aggregate(aggregation, collectionName, JSONObject.class);
        return Lists.newArrayList(out.getMappedResults());
    }

    @Override
    public CommonPage alarmPage(AlarmPageParam alarmPageParam) {
        Query query = new Query();
        String search = ".*?" + alarmPageParam.getSearchValue() + ".*";
        Pattern pattern = Pattern.compile(search, Pattern.CASE_INSENSITIVE);
        query.addCriteria(Criteria.where("alarmLevel").regex(pattern)
                .orOperator(Criteria.where("alarmSourceName").regex(pattern)));
        long count = mongoTemplate.count(query, "alarm");
        if (alarmPageParam.getSkip() != null && alarmPageParam.getLimit() != null) {
            query.skip(alarmPageParam.getSkip()).limit(alarmPageParam.getLimit());
        }
        List<JSONObject> list = mongoTemplate.find(query, JSONObject.class, "alarm");
        return new CommonPage<>(list, null, count);
    }

    private Query buildQueryCondition(List<MongoCondition> conditions) {
        Query query = new Query();
        if (!CollectionUtils.isEmpty(conditions)) {
            conditions.forEach(c -> {
                if ("is".equals(c.getQueryType())) {
                    query.addCriteria(Criteria.where(c.getName()).is(c.getValue()));
                } else if ("in".equals(c.getQueryType())) {
                    query.addCriteria(Criteria.where(c.getName()).in(c.getValue()));
                } else if ("gt".equals(c.getQueryType())) {
                    query.addCriteria(Criteria.where(c.getName()).gt(c.getValue()));
                } else if ("lt".equals(c.getQueryType())) {
                    query.addCriteria(Criteria.where(c.getName()).lt(c.getValue()));
                } else if ("gte".equals(c.getQueryType())) {
                    query.addCriteria(Criteria.where(c.getName()).gte(c.getValue()));
                } else if ("lte".equals(c.getQueryType())) {
                    query.addCriteria(Criteria.where(c.getName()).lte(c.getValue()));
                } else if ("nin".equals(c.getQueryType())) {
                    query.addCriteria(Criteria.where(c.getName()).nin(c.getValue()));
                } else if ("regex".equals(c.getQueryType())) {
                    query.addCriteria(Criteria.where(c.getName()).regex(c.getValue().toString()));
                }
            });
        }
        return query;
    }
}
