package com.yeejoin.amos.iot.netty.gateway;

import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.netty.vehicle.MessageCommonUtil;
import com.yeejoin.amos.iot.netty.vehicle.VehicleFrame;
import com.yeejoin.amos.iot.netty.vehicle.VehicleFrameQueue;
import com.yeejoin.amos.iot.netty.vehicle.VehicleFrameType;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;

/**
 * 中继Lora协议解码器
 * 解码完后，组装成消防协议帧格式的数据，放入队列供后续逻辑处理
 * @author as-mafeipeng
 *
 */
public class UsrChannelHandler extends ChannelInboundHandlerAdapter {
    
    // 最小报文长度8=帧头（1）+地址（2）+产品类型（1）+命令（1）+数据长度（2）+数据（0）+校验（1）
    private static final int MIN_FRAME_LENGTH = 8;
    private static final int LABEL_LENGTH = 8; // 标签长度
    private static final int MIN_LENGTH = 16; // 有人报文最小长度
    private static final byte FRAME_HEADER = (byte) 0x5A; // 帧头
    private static final byte RESPONSE_DATA_COMMAND = (byte) 0x30; // 上位机抄读数据
    private static final byte C0 = (byte) 0xC0; // 转义符
    private static final byte DB = (byte) 0xDB; // 转义符
    private static final byte DD = (byte) 0xDD; // 转义符
    private static final byte DC = (byte) 0xDC; // 转义符
    
    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
        byte[] frame = (byte[]) msg;
        responseIfNeed(ctx, frame);
        //如果是登录帧，必须响应，不然几分钟内会网关会自动断开重连
        if (frame[1] == UsrFrameType.USR_GA_LOGON) {
            String id = genId(frame);
            UsrGatewayManager.record(id, ctx);
        }
        if (frame[1] == UsrFrameType.USR_GA_REPORT_DATA) {
            byte[] slipData = slipData(frame);
            if (slipData.length != 0) {
                byte[] data = slipUnescape(slipData);
                if (checkFrame(data)) {
                    byte[] vFrame = encodeToTabletCommonLabelFrame(data);
                    VehicleFrameQueue.getInstance().add(vFrame);
                } else {
                    Logger.getLogger(this.getClass()).error(
                            "slip转义后的帧校验失败，收到错误帧,未转义:" + MessageCommonUtil.bytes2HexStringWithSpace(slipData)
                                    + ", 转义后：" + MessageCommonUtil.bytes2HexStringWithSpace(data));
                }
            }
        }
        frame = null;
    }
    
    /**
     * 从帧数据中获取网关序列号
     * 序列号长度为11，从第5个字节开始
     * @param frame
     * @return
     */
    private String genId(byte[] frame) {
        int idLength = 0;
        for (int i = 4; i < 15; i++) {
            if (frame[i] == 0x00) {
                break;
            }
            idLength++;
        }
        return new String(MessageCommonUtil.getAnyPartMessage(frame, 4, idLength));
    }
    
    
    /**
     * 给需要响应的帧下发相应的响应
     * @param ctx
     * @param frame
     */
    private void responseIfNeed(ChannelHandlerContext ctx, byte[] frame) {
        byte[] response = UsrGwResponseFrame.createResponse(frame);
        if (response != null) {
            final ByteBuf responsebuf = ctx.alloc().buffer(frame.length);
            responsebuf.writeBytes(response);
            ctx.writeAndFlush(responsebuf);
//            responsebuf.release();
        }
    }

    /**
     * 截取usr协议帧中的数据部分，即中继发送的用slip转义的标签数据
     * @param frame
     * @return
     */
    private byte[] slipData(byte[] frame) {
        if (frame.length > MIN_LENGTH) {
            return MessageCommonUtil.getAnyPartMessage(frame, 15, frame.length - 16);
        }
        return new byte[0];
    }
    
    /**
     * 利用SLIP协议反转义
     * 0xDB,0xDC ==> 0xC0
     * 0xDB,0xDD ==> 0xDB
     * @param slipFrame
     * @return
     */
    private byte[] slipUnescape(byte[] slipFrame) {
        byte[] tmp = new byte[slipFrame.length-2];
        int idx = 0;
        for (int i = 1; i < slipFrame.length - 1; i++, idx++) {
            if (slipFrame[i] == DB) {
                if (slipFrame[i + 1] == DC) {
                    tmp[idx] = C0;
                    i++;
                } else if (slipFrame[i + 1] == DD) {
                    tmp[idx] = DB;
                    i++;
                }
            } else {
                tmp[idx] = slipFrame[i];
            }
        }
        
        if (tmp.length == idx) {
            return tmp;
        }
        return MessageCommonUtil.getAnyPartMessage(tmp, 0, idx);
    }
    
    /**
     * 检查帧是否有效 如果校验和不对，无效
     * 如果帧头不对，无效
     * 如果命令不对，无效
     * 如果数据长度不对，无效
     * 否则，有效
     * 
     * @param frame
     * @return
     */
    private boolean checkFrame(byte[] frame) {
        return checksum(frame) && isHeaderValid(frame) && isCommandValid(frame) && isDataLengthVilad(frame);
    }
    
    /**
     * 和校验， 最后一位为校验和， 校验对象为除校验位以外的所有数据， 取低字节作为校验码
     * 
     * @param frame
     * @return
     */
    private boolean checksum(byte[] frame) {
        if (frame.length < 2) {
            Logger.getLogger(this.getClass()).error("和校验失败：校验数据长度小于2，无法校验");
            return false;
        }

        int sum = 0;
        for (int i = 0; i < frame.length - 1; i++) {
            sum += (frame[i] & 0xff);
        }
        // System.err.println("计算的校验和："+(sum&0xff) +",
        // 原始校验和："+(frame[frame.length - 1] & 0xff));
        if ((sum & 0xff) != (frame[frame.length - 1] & 0xff)) {
            Logger.getLogger(this.getClass()).error("和校验失败，上报的校验和：" + (frame[frame.length - 1] & 0xff) + "，计算的校验和：" + (sum & 0xff));
            return false;
        }
        return true;
    }

    /**
     * 真实帧（反转义后的报文）头是否有效
     * 
     * @param frame
     * @return
     */
    private boolean isHeaderValid(byte[] frame) {
        if (frame[0] != FRAME_HEADER) {
            Logger.getLogger(this.getClass()).error("转义后的帧头错误：" + frame[0]);
            return false;
        }
        return true;
    }

    /**
     * 检验命令是否有效 支持主动上报数据和上位机抄读数据响应
     * 
     * @param frame
     * @return
     */
    private boolean isCommandValid(byte[] frame) {
        byte command = frame[4];
        if (command != RESPONSE_DATA_COMMAND) {
            Logger.getLogger(this.getClass()).error("无效命令：" + command);
            return false;
        }
        return true;
    }

    /**
     * 数据长度是否有效 如果数据长度为0，无效 如果数据长度和真实数据长度不一致，无效 如果数据长度不是标签长度的整数据倍，无效 否则，有效
     * 
     * @param frame
     * @return
     */
    private boolean isDataLengthVilad(byte[] frame) {
        // 第6个字节表示包序号
        // 第7个字节表示数据长度
        int dataLength = MessageCommonUtil.convertByte2Unsigned(frame[6]);
        if (dataLength == 0) {
            Logger.getLogger(this.getClass()).error("数据长度校验失败，数据长度为0");
            return false;
        }

        // 数据长度和实际数据长度不相等
        if (dataLength != dataLength(frame)) {
            Logger.getLogger(this.getClass()).error("数据长度校验失败，数据长度与实际长度不相等");
            return false;
        }
        // 数据长度必须是标签长度的整数据倍
        if (dataLength % LABEL_LENGTH != 0) {
            Logger.getLogger(this.getClass()).warn("数据长度校验失败，数据长度不是标签长度的整数倍");
            return false;
        }
        return true;
    }

    /**
     * 数据长度，slip转义后数据体的长度，即标签数据等的长度
     * 报文长度-最小报文长度
     * @param frame
     * @return
     */
    private int dataLength(byte[] frame) {
        return frame.length - MIN_FRAME_LENGTH;
    }
    
    /**
     * 将中继上报的标签数据，封装成消防协议中平板上报标签类型帧 
     * @param data
     * @return
     */
    private byte[] encodeToTabletCommonLabelFrame(byte[] data) {
        byte[] carCode = MessageCommonUtil.getAnyPartMessage(data, 1, 2);
        byte[] labels = MessageCommonUtil.getAnyPartMessage(data, 7, data.length-8);
        byte[] body = new byte[carCode.length + labels.length + 1];
        System.arraycopy(carCode, 0, body, 0, carCode.length);
        body[carCode.length] = VehicleFrameType.TABLET_ACTIVE_REPORT_COMMON_LABEL;
        System.arraycopy(labels, 0, body, carCode.length+1, labels.length);
        VehicleFrame vFrame = VehicleFrame.createFrameByBody(body);
        if (vFrame == null) {
            return new byte[0];
        }
        return vFrame.getFrame();
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
        cause.printStackTrace();
        ctx.close();
    }


    /**
     * 通道断开时移除通道和设备的关系记录
     */
    @Override
    public void channelUnregistered(ChannelHandlerContext ctx) throws Exception {
        super.channelUnregistered(ctx);
        UsrGatewayManager.remove(ctx);
    }
    
}
