package com.yeejoin.amos.iot.netty.gateway.service;

import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

import org.springframework.stereotype.Service;

import com.yeejoin.amos.iot.netty.gateway.UsrSamplerScheduleTask;
import com.yeejoin.amos.iot.netty.gateway.intfc.IGatewayService;
import com.yeejoin.amos.iot.netty.gateway.param.UsrGatewaySampleTaskParam;

/**
 * 有人网关service
 * @author as-mafeipeng
 *
 */
@Service("gatewayService")
public class GatewayServiceImpl implements IGatewayService {

    /**
     * 单线程任务调度执行器
     */
    private static final ScheduledExecutorService service = Executors.newSingleThreadScheduledExecutor();
    /**
     * 记录采集任务，key为网关序列号，value为已经执行的任务
     */
    private static final Map<String, Set<ScheduledFuture<?>>> tasks = new ConcurrentHashMap<>();

    @Override
    public void addTask(UsrGatewaySampleTaskParam request) {
        String gatewayId = request.getGatewayId();
        List<String> commands = request.getCommands();
        int interval = request.getInterval();
        if (gatewayId != null && commands != null && commands.size() != 0) {
            //先停掉以有任务
            if (tasks.containsKey(gatewayId)) {
                stopTask(gatewayId);
                removeTask(gatewayId);
            }
            
            //总周期=采集间隔*命令数
            int period = interval * commands.size();
            for (int i = 0; i < commands.size(); i++) {
                //按顺序启动任务，每个任务延迟启动时间=采集间隔*命令序号
                int delay = interval * i;
                ScheduledFuture<?> sFuture = service.scheduleAtFixedRate(new UsrSamplerScheduleTask(gatewayId, commands.get(i)), delay, period, TimeUnit.SECONDS);
                recordTask(gatewayId, sFuture);
            }
        }
    }
    
    
    /**
     * 停止任务
     * @param gatewayId
     */
    @Override
    public void stopTask(String gatewayId) {
        if (tasks.containsKey(gatewayId)) {
            tasks.get(gatewayId).forEach(future -> {
                future.cancel(true);
            });
        }
    }

    /**
     * 记录采集任务，方便需要停任务的时候使用
     * @param gatewayId
     * @param sFuture
     */
    private void recordTask(String gatewayId, ScheduledFuture<?> sFuture) {
        if (tasks.containsKey(gatewayId)) {
            Set<ScheduledFuture<?>> set = tasks.get(gatewayId);
            set.add(sFuture);
        } else {
            Set<ScheduledFuture<?>> set = new HashSet<ScheduledFuture<?>>();
            set.add(sFuture);
            tasks.put(gatewayId, set);
        }
    }

    
    /**
     * 移除相关网关任务
     * @param gatewayId
     */
    private void removeTask(String gatewayId) {
        if (tasks.containsKey(gatewayId)) {
            tasks.remove(gatewayId);
        }
    }
}
