package com.yeejoin.amos.iot.netty.mqtt;

import java.util.Date;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.dao.repository.IFrameProtocolDao;
import com.yeejoin.amos.iot.business.dic.ScriptLang;
import com.yeejoin.amos.iot.business.entity.Dic;
import com.yeejoin.amos.iot.business.entity.FrameProtocol;
import com.yeejoin.amos.iot.business.service.intfc.IDicService;
import com.yeejoin.amos.iot.business.service.intfc.IProtocolService;
import com.yeejoin.amos.iot.business.vo.ProtocolVo;
import com.yeejoin.amos.iot.core.util.DateUtil;
import com.yeejoin.amos.iot.entity.ParseResult;
import com.yeejoin.amos.iot.netty.vehicle.VehicleData;
import com.yeejoin.amos.iot.netty.vehicle.VehicleDataQueue;
import com.yeejoin.amos.iot.netty.vehicle.VehicleFrameCache;
import com.yeejoin.amos.iot.netty.vehicle.test.TestVehicleDataQueue;
import com.yeejoin.amos.iot.script.IScriptService;

import net.sf.json.JSONObject;

/**
 * MQTT帧服务实现类
 * 
 * @author tianbo
 *
 */
@Service("mqttFrameService")
public class MqttFrameServiceImpl implements IMqttFrameService {

    @Autowired
    private IProtocolService protocolService;

    @Autowired
    private IDicService dicService;

    @Autowired
    private IScriptService groovyScriptService;

    @Autowired
    private IScriptService javaScriptService;

    @Autowired
    private IFrameProtocolDao frameProtocolDao;

    /**
     * 解析协议数据
     */
    @Override
    public void parseData(byte[] frameBytes) {
        if (frameBytes == null) {
            Logger.getLogger(this.getClass()).warn("frame is null!");
            return;
        }
		String json= new String(frameBytes);
		Map<String, Object> valueMap = (Map<String, Object>) JSONObject.toBean(JSONObject.fromObject(json),Map.class);
		String sid =String.valueOf(valueMap.get("sourceId"));
        int frameType = Integer.parseInt(String.valueOf(valueMap.get("frameType")));

        // 2.调用groovy/javascript代码解析帧体中的数据
        FrameProtocol frameProtocol = frameProtocolDao.findByFrameType(frameType);
        if(frameProtocol == null){
            Logger.getLogger(this.getClass()).warn("frameProtocol is null!");
            return;
        }
        Long protocolId = frameProtocol.getProtocolId();
        if (protocolId == null) {
            Logger.getLogger(this.getClass()).warn("protocolId is null!");
            return;
        }
        ProtocolVo protocolVo = protocolService.findById(protocolId);
        Dic dic = dicService.getById(String.valueOf(protocolVo.getDicIdScriptLang()));
        // 调用groovy/javascript代码解析协议数据
        Object[] args = new Object[]{frameBytes,sid};
        ParseResult parseResult = null;
        switch (dic.getCode()) {
            case ScriptLang.GROOVY:
                // groovy解析协议数据
                parseResult = groovyScriptService.analyze(protocolVo.getScriptCode(), args);
                break;
            case ScriptLang.JAVASCRIPT:
                // javascript解析协议数据
                parseResult = javaScriptService.analyze(protocolVo.getScriptCode(), args);
                break;
            case ScriptLang.PYTHON:
                // python解析协议数据---暂未实现
                break;
            case ScriptLang.LUA:
                // lua解析协议数据---暂未实现
                break;
            default:
                // 不执行任何业务
                break;
        }
        String resultJson = null;
        try {
            resultJson = JSONObject.fromObject(parseResult.getResult()).toString();
        } catch (Exception e) {
            Logger.getLogger(this.getClass()).error("protocol analye result error!" + e);
            return;
        }
        if ((resultJson != null) && resultJson.contains("errorMessage")) {
            JSONObject json1 = JSONObject.fromObject(resultJson);
            Object type = json1.get("type");
            VehicleFrameCache.getInstance().unnormalFrame(sid, String.valueOf(frameType), type);
            Logger.getLogger(this.getClass()).info("协议解析异常,sid:" + sid);
            return;
        }
        if (StringUtils.isNotEmpty(resultJson)) {
            JSONObject json1 = JSONObject.fromObject(resultJson);
            VehicleData vehicleData = new VehicleData();
            vehicleData.setSourceID(json1.getString("sourceId"));
            vehicleData.setFrameType((byte)Integer.parseInt(json1.getString("frameType")));
            Object type = json1.get("type");
            if (type != null) {
                if (type instanceof Byte) {
                    vehicleData.setLabelType((Byte) type);
                } else if (type instanceof Integer) {
                    Integer v = (Integer) type;
                    vehicleData.setLabelType(v.byteValue());
                }
            }
            vehicleData.setFrameJson(json1.toString());
            Date now = new Date();
            Date exeTime = null;
            try {
                if (StringUtils.isEmpty(json1.getString("formatTime"))) {
                    exeTime = now;// 当前时间
                } else {
                    exeTime = DateUtil.str2Date(json1.getString("formatTime"), IotConstant.DATE_TIME_STR_YMDHMS);
                }
            } catch (Exception e) {
                Logger.getLogger(this.getClass()).info("formatTime not exists!");
                exeTime = now;// 当前时间
            }
            if (exeTime == null) {
                exeTime = now;
            }
            vehicleData.setTime(exeTime);
            boolean isValid = isFrameValid(json1);
            // 解析结果数据放入队列
            if (isValid) {
                VehicleDataQueue.getInstance().add(vehicleData);
                VehicleFrameCache.getInstance().normalFrame(sid, String.valueOf(frameType), type);
                //放入设备测试队列
                TestVehicleDataQueue.getInstance().add(vehicleData);
            } else {
                Logger.getLogger(this.getClass()).warn("isValid=fale,:" + resultJson);
            }
        }
    }

    /**
     * 监测帧数据是否有效
     *
     * @param json
     * @return
     */
    private boolean isFrameValid(JSONObject json) {
        boolean isValid = true;
        try {
            String status = (String) json.get("status");
            if ("信息不可用".equals(status)) {
                isValid = false;
            }
        } catch (Exception e) {
            Logger.getLogger(this.getClass()).error(e);
        }
        return isValid;
    }
}
