package com.yeejoin.amos.iot.netty.vehicle;

import java.util.List;

import org.apache.log4j.Logger;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.ByteToMessageDecoder;

/**
 * 车载终端自定义协议帧解码器 从tcp字节流中按协议截取完整的帧
 * 
 * @author as-mafeipeng
 *
 */
public class VehicleFrameDecoder extends ByteToMessageDecoder {

	@Override
	protected void decode(ChannelHandlerContext ctx, ByteBuf in, List<Object> out) throws Exception {

		if (isHeardReady(in) && isBodyReady(in) && isTailReady(in)) {
			byte[] frame = new byte[lenthOfFrame(in)];
			in.readBytes(frame);
			if (crcCheck(frame)) {
				out.add(frame);
				return;
			}
		}

	}

	/**
	 * 检查帧头是否就位，长度够，且格式正确
	 * 
	 * @param in
	 * @return
	 */
	private boolean isHeardReady(ByteBuf in) {
		if (in.readableBytes() < VehicleFrame.HEAD_LENGTH) {
			return false;
		}

		if (in.getByte(in.readerIndex() + 0) != VehicleFrame.START
				|| in.getByte(in.readerIndex() + 3) != VehicleFrame.START) {
			// 如果帧头格式不对，丢弃当前可读的字节
			discardReadbleBytes(in);
			return false;
		}

		return true;
	}

	/**
	 * 检查帧体是否就位，长度够
	 * 
	 * @param in
	 * @return
	 */
	private boolean isBodyReady(ByteBuf in) {
		return in.readableBytes() >= VehicleFrame.HEAD_LENGTH + lengthOfBody(in);
	}

	/**
	 * 检查帧尾是否就位，长度够，且格式正确
	 * 
	 * @param in
	 * @return
	 */
	private boolean isTailReady(ByteBuf in) {
		int lengthOfFrame = lenthOfFrame(in);

		if (in.readableBytes() < lengthOfFrame) {
			return false;
		}
		if (in.getByte(in.readerIndex() + lengthOfFrame - 1) != VehicleFrame.END) {
			// 帧格式不正确，丢弃当前可读的字节
			discardReadbleBytes(in);
			return false;
		}

		return true;
	}

	/**
	 * 丢弃当前缓存中可读的字节 通过移动readerIndex实现
	 * 
	 * @param in
	 */
	private void discardReadbleBytes(ByteBuf in) {
		in.readerIndex(in.readerIndex() + in.readableBytes());
	}

	/**
	 * 整个帧的长度
	 * 
	 * @param in
	 * @return
	 */
	private int lenthOfFrame(ByteBuf in) {
		return VehicleFrame.HEAD_LENGTH + lengthOfBody(in) + VehicleFrame.TAIL_LENGTH;
	}

	/**
	 * 帧体的长度
	 * 
	 * @param in
	 * @return
	 */
	private int lengthOfBody(ByteBuf in) {
		return in.getUnsignedShort(in.readerIndex() + 1);
	}

	/**
	 * 帧体的长度
	 * 
	 * @param frame
	 * @return
	 */
	private int lengthOfBody(byte[] frame) {
		return MessageCommonUtil.convertTwoBytes2Uint16(frame[2], frame[1]);
	}

	/**
	 * crc校验，帧尾前两个自己为crc校验结果，高位在前，校验数据为除了帧尾的其他所有字节数据，即帧头+帧体
	 * 
	 * @param frame
	 * @return
	 */
	private boolean crcCheck(byte[] frame) {
		int lengthOfCrcData = VehicleFrame.HEAD_LENGTH + lengthOfBody(frame);
		byte[] crcData = new byte[lengthOfCrcData];
		System.arraycopy(frame, 0, crcData, 0, lengthOfCrcData);
		byte[] crc = MessageCommonUtil.hexStr2ByteArray(ComputeCRCUtil.calcCrc16(crcData));
		return ((crc[0] == frame[frame.length - 2]) && (crc[1] == frame[frame.length - 3])); // 交叉比较是因为计算的crc是小端，而上报的crc是大端
	}

}
