package com.yeejoin.amos.iot.rule.engine.service.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;

import org.apache.log4j.Logger;
import org.kie.api.KieServices;
import org.kie.api.builder.KieBuilder;
import org.kie.api.builder.KieFileSystem;
import org.kie.api.builder.Results;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.rule.FactHandle;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.yeejoin.amos.iot.business.entity.DroolsRule;
import com.yeejoin.amos.iot.business.entity.RuleGroup;
import com.yeejoin.amos.iot.business.service.intfc.IDroolsRuleService;
import com.yeejoin.amos.iot.business.service.intfc.INodeVoService;
import com.yeejoin.amos.iot.business.service.intfc.IRuleGroupService;
import com.yeejoin.amos.iot.rule.engine.service.IDroolsManageService;
import com.yeejoin.amos.iot.rule.resource.RuleAIEntity;

/**
 * <pre>
 * drools引擎管理
 * </pre>
 *
 * @author zq
 * @version $Id: DroolsManage.java, v 0.1 2017年12月27日 下午8:39:01 zq Exp $
 */
@Service("DroolsManageService")
public class DroolsManageServiceImpl implements IDroolsManageService
{
	private static final Logger log = Logger
			.getLogger(DroolsManageServiceImpl.class);
	
	@Autowired
	IRuleGroupService ruleGroupService;
	
	@Autowired
	IDroolsRuleService droolsRuleService;
	
	KieServices kieServices;
	
	@Autowired
	private INodeVoService nodeVoService;
	
	/**
	 * <pre>
	 * key rulegroup.id+rulegroup.name
	 * </pre>
	 */
	private Map<String, KieContainer> kieContainerMap;
	
	@PostConstruct
	public void init(){
		kieServices = KieServices.Factory.get();
		initKieContainerMap();
	}
	
	/**
	 * <pre>
	 * 通过rulegroup初始化kiecontainerMap,一个rulegroup对应一个kiecontainer
	 * </pre>
	 *
	 */
	private void initKieContainerMap(){
		kieContainerMap = new HashMap<String,KieContainer>();
		//从规则库中获取所有规则组，按规则组生成kiecontainer，用于执行规则
		List<RuleGroup> ruleGroupList = ruleGroupService.findAllByIsUser(true);
		KieContainer kieContainer;
		List<DroolsRule> droolsRuleList;
		for (RuleGroup ruleGroup : ruleGroupList)
		{
			droolsRuleList = droolsRuleService.findByGroupIdAndIsUser(ruleGroup.getId(), true);
			if(droolsRuleList.size()>0){
				kieContainer = null;
				try
				{
					kieContainer = initKieContainer(ruleGroup,droolsRuleList);
				}
				catch (IllegalStateException e)
				{
					log.error(ruleGroup.getDisplayName()+"规则组初始化失败，错误信息："+e.getMessage());
				}
				
				if(kieContainer!=null)
				{
					kieContainerMap.put(ruleGroup.getId()+"_"+ruleGroup.getName(), kieContainer);
				}
			}
		}
	}
	
	/**
	 * <pre>
	 * 通过rulegroup初始化kiecontainer
	 * </pre>
	 *
	 * @param ruleGroup
	 * @return
	 */
	private KieContainer initKieContainer(RuleGroup ruleGroup,List<DroolsRule> ruleList) throws IllegalStateException{
		//生成kie文件服务，用于加载drools规则数据
        KieFileSystem kfs = kieServices.newKieFileSystem();
		//将drools规则写入kie文件服务
		for (DroolsRule droolsRule : ruleList)
		{
			kfs.write("src/main/resources/rules/"+droolsRule.getId()+".drl", droolsRule.getDroolsRule().getBytes());
		}
		//生成kiebuilder,编译drools规则
		KieBuilder kieBuilder = kieServices.newKieBuilder(kfs).buildAll();
		//drools规则编译结结果
		Results results = kieBuilder.getResults();
		//如果编译错误，抛出错误信息
        if (results.hasMessages(org.kie.api.builder.Message.Level.ERROR)) {
            throw new IllegalStateException(results.toString());
        }
        //编译成功，生成kiecontainer
		return kieServices.newKieContainer(kieServices.getRepository().getDefaultReleaseId());
	}

	@Override
	public void fireRule(RuleAIEntity ruleAIEntity)
	{
		KieContainer kieContainer = kieContainerMap.get("1_AlarmRuleGroup");
		if (kieContainer != null)
		{
			fireDroolsRule(kieContainer,ruleAIEntity);
		}
	}

	@Override
	public void updateRule(RuleGroup ruleGroup,List<DroolsRule> ruleList) throws IllegalStateException{
		String groupId = ruleGroup.getId();
		if(ruleList.size()==0){//规则组内规则为0，销毁kieContainer
			destoryKieContainer(groupId);
		}else{
			//根据规则组和组内规则，生成kieContainer
			KieContainer kieContainer = initKieContainer(ruleGroup, ruleList);
			//销毁规则组上一个KieContainer
			destoryKieContainer(groupId);
			
			kieContainerMap.put(ruleGroup.getId()+"_"+ruleGroup.getName(), kieContainer);
		}
		
	}
	
	/**
	 * <pre>
	 * 销毁规则组上一个KieContainer
	 * </pre>
	 *
	 * @param groupId
	 */
	private void destoryKieContainer(String groupId){
		KieContainer oldKieContainer = null;
		String oldKey = null;
		//根据groupId获取规则组的KieContainer
		for (Map.Entry<String, KieContainer> entry : kieContainerMap.entrySet()) {  
			if(entry.getKey().contains(groupId+"_"))
			{
				oldKieContainer = entry.getValue();
				oldKey = entry.getKey();
			}
		} 
		
		if(oldKieContainer != null){
			//销毁oldKieContainer
			oldKieContainer.dispose();
			kieContainerMap.remove(oldKey);
		}
	}
	
	/**
	 * <pre>
	 * 根据fact和对应的kieContainer,触发规则
	 * </pre>
	 *
	 * @param kieContainer
	 * @param ruleAIEntity
	 */
	private void fireDroolsRule(KieContainer kieContainer,RuleAIEntity ruleAIEntity)
	{
		
			//获取kiesession，用于执行规则
			KieSession kieSession = kieContainer.newKieSession();
			//将触发因子(fact)插入kiesession中，用于匹配要执行的规则
			FactHandle fact = kieSession.insert(ruleAIEntity);
			try
			{
				//触发规则
				int count = kieSession.fireAllRules();
			}
			catch (Exception e)
			{
				log.error("执行规则错误："+e.getMessage());
			}
			
			//释放facthandle
			kieSession.delete(fact);
			//释放kiesession
			kieSession.dispose();
		
		
	}
	
	@Override
	public void destroyRule(List<RuleGroup> ruleGroupList)
	{
		for (RuleGroup ruleGroup : ruleGroupList)
		{
			destoryKieContainer(ruleGroup.getId());
		}
	}

}
